@@ -1,5 +1,6 @@
 from kirin import types
-from kirin.prelude import basic
+from kirin.prelude import basic, structural
+from kirin.dialects import ilist
 
 
 @basic
@@ -28,3 +29,16 @@ def test_inter_method_infer():
     assert foo.arg_types[0] == types.Int
     assert foo.inferred is False
     assert foo.return_type is types.Any
+
+
+def test_method_constant_type_infer():
+
+    @structural(typeinfer=True, fold=False)
+    def _new(qid: int):
+        return 1
+
+    @structural(fold=False, typeinfer=True)
+    def alloc(n_iter: int):
+        return ilist.map(_new, ilist.range(n_iter))
+
+    assert alloc.return_type.is_subseteq(ilist.IListType[types.Literal(1), types.Any])
